//
//  slideshow.js
//  TangoMe, Inc.
//
//  Created by Quoc Le on 5/1/14.
//  Copyright (c) 2014 Tango.me. All rights reserved.
//

window.spotango = {
	version: '1.0',
	transition : {}
};

function getRandomInt(min, max) {
  return Math.floor(Math.random() * (max - min + 1)) + min;
}

(function($){

	spotango.slideshow = function(elem, opts) {
		this.options = $.extend({
			transition : "fadeOut",
			base_url : "",
			play_button_enabled: true,
			time_per_slide : 2700,
			native_player : false,
			auto_play : false,
			max_play_sec : -1 // disable
		}, opts);

		this.element = $(elem);

		this.slide_playing = false;
		this.music_playing = false;

		this.photos = [];
		this.base_url = this.options.base_url;

		//this.meta = this.options.meta;
		var img1 = document.createElement( "div" );
		var img2 = document.createElement( "div" );

		this.img1 = $(img1);
		this.img2 = $(img2);

		this.top = this.img1.attr('id', "slide");
		this.bottom = this.img2.attr('id', "slide");

		this.bottom.addClass("slide").css("z-index", -1);
		this.top.addClass("slide").css("z-index", 10);

		this.element.append(img1);
		this.element.append(img2);		

		this.randomTransitions = ["circleOut", "bounceOutLeft", "fadeOutDown", "fadeOutUp", "rotateOutUpLeft", "slideOutLeft", "slideOutRight"];		

		if (this.options.play_button_enabled) {
			var obj = this;
			$("#play_btn").click(function() {
			 	console.log("clicked");
			 	if(obj.music_playing) {
				 	obj.stopMusic();	
				 	if(!obj.options.auto_play) {
				 		obj.stop();
				 	}
			 	} else {
				 	obj.startMusic();		 		
				 	if(!obj.options.auto_play) {
				 		obj.start();
				 	}
			 	}
			 });
		} else {
			$("#play_icon").css("display", "none");	
		}
	};

	spotango.slideshow.prototype = {
		
		constructor: spotango.slideshow,

		loadMeta: function(meta_) {
			this.meta = meta_;

			this.top.removeClass().css("z-index", 10);
			this.bottom.removeClass().css("z-index", -1);

			this.setImage(this.top, this.base_url + this.meta.images[0]);

			if (this.meta.images.length > 1) {
				this.setImage(this.bottom, this.base_url + this.meta.images[1]);
			}
			this.preloaded = true;

			//console.log("load meta ", meta_);
			return true;
		},

		setImage: function(el, url) {
			el.css("background-image", 'url("'+ url +'")');
			//console.log("url = ", url);
		},

		startMusic: function() {
			console.log("start() track = ", this.meta.track_url,  " max = ", this.options.max_play_sec);
			this.audio = new Audio(this.meta.track_url);		
			this.audio.play();
			this.audio.volume = 0;
			$(this.audio).animate({volume: 1}, 1000);	

			this.music_playing = true;
			var self = this;

			var onMusicEnded = function() {
				console.log("end of music.");				
				var musicEnded = self.options.max_play_sec > 0 ? false : true ;
				self.stopMusic(musicEnded);
			 	if(!self.options.auto_play) {
			 		self.stop();
			 	}				
			}

			$(this.audio).on("ended", onMusicEnded);			

			$(this.audio).on("timeupdate", function() {
				var w =  (this.currentTime*100) / (self.options.max_play_sec > 0 ? self.options.max_play_sec : this.duration);
				$("#progress").width(w + "%");
			});			

			if(self.options.max_play_sec > 0) {
				self.musicEndTimer = setInterval(onMusicEnded, self.options.max_play_sec * 1000);
			}

    		$("#progress").show();

			if (this.options.play_button_enabled) {
				$(("#play_icon")).css("display", "none");	 		
			}				
		},

		stopMusic: function(musicEnded) {
			console.log("stopMusic() "  + musicEnded);

			if(this.music_playing == false) {
				console.log("music already stopped!");
				return;
			} else {
				this.music_playing = false;				
			}

			if (this.options.play_button_enabled) {
				$(("#play_icon")).css("display", "block");	 		
			}

			if (!musicEnded) {
				this.audio.pause();
    			//this.audio.currentTime = 0;
    		}

    		if(this.options.native_player) {
	    		window.location = "spotango_player://action=stop";  // this calls to native code
    		}
    		
    		$("#progress").hide();
			$("#progress").width("0%");

			clearInterval(this.audioFade);				
			clearInterval(this.progressBar);

			//clearInterval(this.animateTimer);				
		},

		start: function() {

			if(this.slide_playing) {
				console.log("Already started!");
				return;
			} else {
				this.slide_playing = true;				
			}

		 	if(!this.options.auto_play && !this.preloaded) {
		 		this.loadMeta(this.meta);
		 	}

		 	this.preloaded = false;		 	
			var index = 0;

			var self = this;
			var onFinish = function() {

				if(self.slide_playing == false) {
					return;
				}

				var tmp = self.top;
				self.top = self.bottom;
				self.bottom = tmp;

				self.top.css("z-index", 10);
				self.bottom.removeClass().css("z-index", -1);

				var nextImageIndex = (index+1)%self.meta.images.length;
				self.setImage(self.bottom, self.base_url + self.meta.images[nextImageIndex]);
				//console.log(new Date()- startTime, "current = ", index, " next = ", nextImageIndex , " url = ", self.meta.images[nextImageIndex]);
				index++;
			}

			if(this.audioFade != null) {	
				clearInterval(this.audioFade);			
			}

			if(this.animateTimer != null) {	
				clearInterval(this.animateTimer);
			}

			if (this.meta.images.length > 1) {
				this.animateTimer = setInterval(function() {
					var transition = self.options.transition;
					if(transition == "random") {
						transition = self.randomTransitions[getRandomInt(0,self.randomTransitions.length-1)];
					}

					spotango.transition.factory(self.top, self.bottom, transition, onFinish, self.meta)

				}, this.options.time_per_slide);
			}
		},

		stop : function() {
			if(this.slide_playing == false) {
				console.log("slide already stopped.");
				return;
			} else {
				this.slide_playing = false;				
			}

			clearInterval(this.animateTimer);							
		}
	};

	// factory for animations
	spotango.transition.factory = function(top, bottom, transition, finish, meta) {
		if(transition == "photoWallZoomIn") {
			return spotango.transition.photoWallZoomIn(top, bottom, transition, finish, meta);
		} else {
			return spotango.transition.standard(top, bottom, transition, finish);
		}
	};	

	// circle masking out
	spotango.transition.standard = function(top, bottom, transition, finish) {
		//console.log("Transition " + transition);
		top.removeClass().addClass(transition + ' animated')
			.one('webkitAnimationEnd mozAnimationEnd MSAnimationEnd oanimationend animationend', finish);
	};

	// this is proxy for animate.css
	spotango.transition.photoWallZoomIn = function(top, bottom, transition, finish, meta) {

		var div = $(document.createElement( "div" ));
		for (var i=0; i < meta.images.length; i++) {
			div.append("<img class=\"small-thumbnail\" src=\"/" + meta.images[i] + "\"/>");
		}
		div.css("background-image", "none");
		div.css("background-color", "black");

		top.append(div);

		div.transition({ x: 300 }, 1000, 'linear', function() {
			top.empty();
			finish();
		});

		// top.removeClass().addClass('zoomIn animated')
		// 	.one('webkitAnimationEnd mozAnimationEnd MSAnimationEnd oanimationend animationend', function() {
		// 		console.log("zoomInLong done.");
		// 		top.empty();
		// 		finish();
		// 	});
	};

})(window.jQuery || window.Zepto);

